package com.aizuda.easy.retry.server.common.enums;

import com.aizuda.easy.retry.server.common.exception.EasyRetryServerException;
import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * job 触发器类型枚举
 *
 * @author: xiaowoniu
 * @date : 2023-12-06 17:21
 * @since : 2.5.0
 */
@Getter
@AllArgsConstructor
public enum JobTaskExecutorSceneEnum {
    AUTO_JOB(1, TaskTypeEnum.JOB),
    MANUAL_JOB(2,  TaskTypeEnum.JOB),
    AUTO_WORKFLOW(3, TaskTypeEnum.WORKFLOW),
    MANUAL_WORKFLOW(4, TaskTypeEnum.WORKFLOW),
    ;

    private final Integer type;
    private final TaskTypeEnum taskType;

    /**
     * 根据给定的类型获取对应的触发器类型枚举
     *
     * @param type 触发器类型的整数值
     * @return 对应的触发器类型枚举
     * @throws EasyRetryServerException 当给定的类型不是有效的枚举类型时抛出异常
     */
    public static JobTaskExecutorSceneEnum get(Integer type) {
        for (JobTaskExecutorSceneEnum jobTaskExecutorSceneEnum : JobTaskExecutorSceneEnum.values()) {
            if(jobTaskExecutorSceneEnum.getType().equals(type)) {
                return jobTaskExecutorSceneEnum;
            }
        }

        throw new EasyRetryServerException("无效枚举类型.[{}]", type);
    }

}
